@@ -40,17 +40,21 @@ import logging
 import netrc
 import tempfile
 import shutil
+import threading
+import signal
 
 try:
     # py3k
     from urllib.request import urlopen, HTTPPasswordMgrWithDefaultRealm, \
         HTTPBasicAuthHandler, build_opener
     from urllib.parse import urlparse
+    from queue import Queue
 except ImportError:
     # py2.7
     from urlparse import urlparse
     from urllib2 import urlopen, HTTPPasswordMgrWithDefaultRealm, \
         HTTPBasicAuthHandler, build_opener
+    from Queue import Queue
 
 from vcstools.vcs_base import VcsError
 
@@ -68,7 +72,7 @@ def ensure_dir_notexists(path):
         return True
     except OSError as ose:
         # ignore if directory
-        if not ose.errno in [errno.ENOENT, errno.ENOTEMPTY, errno.ENOTDIR]:
+        if ose.errno not in [errno.ENOENT, errno.ENOTEMPTY, errno.ENOTDIR]:
             return False
 
 
@@ -141,7 +145,7 @@ def _netrc_open(uri, filename=None):
     try:
         info = netrc.netrc(filename).authenticators(machine)
         if info is not None:
-            (username, _ , password) = info
+            (username, _, password) = info
             if username and password:
                 pass_man = HTTPPasswordMgrWithDefaultRealm()
                 pass_man.add_password(None, machine, username, password)
@@ -197,7 +201,8 @@ def sanitized(arg):
     # this also detects some false positives, like bar"";foo
     if '"' in arg:
         if (len(shlex.split(safe_arg, False, False)) != 1):
-            raise VcsError("Shell injection attempt detected: >%s< = %s"%(arg, shlex.split(safe_arg, False, False)))
+            raise VcsError("Shell injection attempt detected: >%s< = %s" %
+                           (arg, shlex.split(safe_arg, False, False)))
     return safe_arg
 
 
@@ -213,7 +218,7 @@ def _discard_line(line):
                         "Installing", "Using ",
                         "No ", "Tree ",
                         "All ",
-                        "+N  ", "-D  ", " M  ", " M* ", "RM" # bzr
+                        "+N  ", "-D  ", " M  ", " M* ", "RM"  # bzr
                         ]
     for pre in discard_prefixes:
         if line.startswith(pre):
@@ -221,53 +226,17 @@ def _discard_line(line):
     return False
 
 
-def run_shell_command(cmd, cwd=None, shell=False, us_env=True,
-                      show_stdout=False, verbose=False,
-                      no_warn=False, no_filter=False):
-    """
-    executes a command and hides the stdout output, loggs stderr
-    output when command result is not zero. Make sure to sanitize
-    arguments in the command.
-
-    :param cmd: A string to execute.
-    :param shell: Whether to use os shell.
-    :param us_env: changes env var LANG before running command, can influence program output
-    :param show_stdout: show some of the output (except for discarded lines in _discard_line()), ignored if no_filter
-    :param no_warn: hides warnings
-    :param verbose: show all output, overrides no_warn, ignored if no_filter
-    :param no_filter: does not wrap stdout, so invoked command prints everything outside our knowledge
-    this is DANGEROUS, as vulnerable to shell injection.
-    :returns: ( returncode, stdout, stderr); stdout is None if no_filter==True
-    :raises: VcsError on OSError
-    """
-    try:
-        env = copy.copy(os.environ)
-        if us_env:
-            env ["LANG"] = "en_US.UTF-8"
-        if no_filter:
-            # in no_filter mode, we cannot pipe stdin, as this
-            # causes some prompts to be hidden (e.g. mercurial over
-            # http)
-            stdout_target = None
-            stderr_target = None
-        else:
-            stdout_target = subprocess.PIPE
-            stderr_target = subprocess.PIPE
-        proc = subprocess.Popen(cmd,
-                                shell=shell,
-                                cwd=cwd,
-                                stdout=stdout_target,
-                                stderr=stderr_target,
-                                env=env)
-        # when we read output in while loop, it would not be returned
-        # in communicate()
-        stdout_buf = []
-        stderr_buf = []
-        if not no_filter and (verbose or show_stdout):
-            # this loop runs until proc is done
-            # it listen to the pipe, print and stores result in buffer for returning
-            # this allows proc to run while we still can filter out output
-            # avoiding readline() because it may block forever
+def _read_shell_output(proc, no_filter, verbose, show_stdout, output_queue):
+    # when we read output in while loop, it would not be returned
+    # in communicate()
+    stdout_buf = []
+    stderr_buf = []
+    if not no_filter:
+        if (verbose or show_stdout):
+            # this loop runs until proc is done it listen to the pipe, print
+            # and stores result in buffer for returning this allows proc to run
+            # while we still can filter out output avoiding readline() because
+            # it may block forever
             for line in iter(proc.stdout.readline, b''):
                 line = line.decode('UTF-8')
                 if line is not None and line != '':
@@ -285,8 +254,80 @@ def run_shell_command(cmd, cwd=None, shell=False, us_env=True,
                     stderr_buf.append(line)
                 if not line:
                     break
+    output_queue.put(proc.communicate())
+    output_queue.put(stdout_buf)
+    output_queue.put(stderr_buf)
+
+
+def run_shell_command(cmd, cwd=None, shell=False, us_env=True,
+                      show_stdout=False, verbose=False, timeout=None,
+                      no_warn=False, no_filter=False):
+    """
+    executes a command and hides the stdout output, loggs stderr
+    output when command result is not zero. Make sure to sanitize
+    arguments in the command.
+
+    :param cmd: A string to execute.
+    :param shell: Whether to use os shell.
+    :param us_env: changes env var LANG before running command, can influence program output
+    :param show_stdout: show some of the output (except for discarded lines in _discard_line()), ignored if no_filter
+    :param no_warn: hides warnings
+    :param verbose: show all output, overrides no_warn, ignored if no_filter
+    :param timeout: time allocated to the subprocess
+    :param no_filter: does not wrap stdout, so invoked command prints everything outside our knowledge
+    this is DANGEROUS, as vulnerable to shell injection.
+    :returns: ( returncode, stdout, stderr); stdout is None if no_filter==True
+    :raises: VcsError on OSError
+    """
+    try:
+        env = copy.copy(os.environ)
+        if us_env:
+            env["LANG"] = "en_US.UTF-8"
+        if no_filter:
+            # in no_filter mode, we cannot pipe stdin, as this
+            # causes some prompts to be hidden (e.g. mercurial over
+            # http)
+            stdout_target = None
+            stderr_target = None
+        else:
+            stdout_target = subprocess.PIPE
+            stderr_target = subprocess.PIPE
+
+        # additional parameters to Popen when using a timeout
+        crflags = {}
+        if timeout is not None:
+            if hasattr(os.sys, 'winver'):
+                crflags['creationflags'] = subprocess.CREATE_NEW_PROCESS_GROUP
+            else:
+                crflags['preexec_fn'] = os.setsid
+
+        proc = subprocess.Popen(cmd,
+                                shell=shell,
+                                cwd=cwd,
+                                stdout=stdout_target,
+                                stderr=stderr_target,
+                                env=env,
+                                **crflags)
+
+        # using a queue to enable usage in a separate thread
+        q = Queue()
+        if timeout is None:
+            _read_shell_output(proc, no_filter, verbose, show_stdout, q)
+        else:
+            t = threading.Thread(target=_read_shell_output,
+                                 args=[proc, no_filter, verbose, show_stdout, q])
+            t.start()
+            t.join(timeout)
+            if t.isAlive():
+                if hasattr(os.sys, 'winver'):
+                    os.kill(proc.pid, signal.CTRL_BREAK_EVENT)
+                else:
+                    os.killpg(proc.pid, signal.SIGTERM)
+                t.join()
+        (stdout, stderr) = q.get()
+        stdout_buf = q.get()
+        stderr_buf = q.get()
 
-        (stdout, stderr) = proc.communicate()
         if stdout is not None:
             stdout_buf.append(stdout.decode('utf-8'))
         stdout = "\n".join(stdout_buf)
@@ -296,10 +337,10 @@ def run_shell_command(cmd, cwd=None, shell=False, us_env=True,
         message = None
         if proc.returncode != 0 and stderr is not None and stderr != '':
             logger = logging.getLogger('vcstools')
-            message = "Command failed: '%s'"%(cmd)
+            message = "Command failed: '%s'" % (cmd)
             if cwd is not None:
-                message += "\n run at: '%s'"%(cwd)
-            message += "\n errcode: %s:\n%s"%(proc.returncode, stderr)
+                message += "\n run at: '%s'" % (cwd)
+            message += "\n errcode: %s:\n%s" % (proc.returncode, stderr)
             logger.warn(message)
         result = stdout
         if result is not None:
@@ -307,6 +348,6 @@ def run_shell_command(cmd, cwd=None, shell=False, us_env=True,
         return (proc.returncode, result, message)
     except OSError as ose:
         logger = logging.getLogger('vcstools')
-        message = "Command failed with OSError. '%s' <%s, %s>:\n%s"%(cmd, shell, cwd, ose)
+        message = "Command failed with OSError. '%s' <%s, %s>:\n%s" % (cmd, shell, cwd, ose)
         logger.error(message)
         raise VcsError(message)
