@@ -44,7 +44,6 @@ import sys
 import gzip
 
 import dateutil.parser  # For parsing date strings
-import xml.dom.minidom  # For parsing logfiles
 
 from vcstools.vcs_base import VcsClientBase, VcsError
 from vcstools.common import sanitized, normalized_rel_path, run_shell_command
@@ -66,7 +65,7 @@ def _get_hg_version():
     return version
 
 
-#hg diff cannot seem to be persuaded to accept a different prefix for filenames
+# hg diff cannot seem to be persuaded to accept a different prefix for filenames
 def _hg_diff_path_change(diff, path):
     """
     Parses hg diff result and changes the filename prefixes.
@@ -139,11 +138,12 @@ class HgClient(VcsClientBase):
             return output.rstrip()
         return None
 
-    def detect_presence(self):
-        return (self.path_exists() and
-                os.path.isdir(os.path.join(self._path, '.hg')))
+    @staticmethod
+    def static_detect_presence(path):
+        return os.path.isdir(os.path.join(path, '.hg'))
 
-    def checkout(self, url, version='', verbose=False, shallow=False):
+    def checkout(self, url, version='', verbose=False,
+                 shallow=False, timeout=None):
         if url is None or url.strip() == '':
             raise ValueError('Invalid empty url : "%s"' % url)
         # make sure that the parent directory exists for #3497
@@ -164,16 +164,16 @@ class HgClient(VcsClientBase):
         if version is not None and version.strip() != '':
             cmd = "hg checkout %s" % sanitized(version)
             value, _, msg = run_shell_command(cmd,
-                                            cwd=self._path,
-                                            shell=True,
-                                            no_filter=True)
+                                              cwd=self._path,
+                                              shell=True,
+                                              no_filter=True)
             if value != 0:
                 if msg:
                     sys.stderr.write('%s\n' % msg)
                 return False
         return True
 
-    def update(self, version='', verbose=False):
+    def update(self, version='', verbose=False, timeout=None):
         verboseflag = ''
         if verbose:
             verboseflag = '--verbose'
@@ -216,8 +216,8 @@ class HgClient(VcsClientBase):
                                                      cwd=self._path,
                                                      us_env=True)
                     if (output.strip() != ''
-                        and not output.startswith("abort")
-                        or repeated is True):
+                            and not output.startswith("abort")
+                            or repeated is True):
 
                         matches = [l for l in output.splitlines() if l.startswith('changeset: ')]
                         if len(matches) == 1:
@@ -251,7 +251,7 @@ class HgClient(VcsClientBase):
     def get_log(self, relpath=None, limit=None):
         response = []
 
-        if relpath == None:
+        if relpath is None:
             relpath = ''
 
         if self.path_exists() and os.path.exists(os.path.join(self._path, relpath)):
@@ -273,7 +273,6 @@ class HgClient(VcsClientBase):
                 response = response_str.strip('\n\x1e').split("\x1e")
                 response = [row.strip().split("\x1f") for row in response]
                 response = [dict(zip(HG_COMMIT_FIELDS, row)) for row in response]
-                print(response)
                 # Parse dates
                 for entry in response:
                     entry['date'] = dateutil.parser.parse(entry['date'])
