@@ -48,7 +48,7 @@ class VcsError(Exception):
     violated assumption"""
 
     def __init__(self, value):
-        super(VcsError, self).__init__(self)
+        super(VcsError, self).__init__(value)
         self.value = value
 
     def __str__(self):
@@ -127,48 +127,65 @@ class VcsClientBase(object):
           spec is provided, the respective revision number.
         :rtype: str
         """
-        raise NotImplementedError("Base class get_version method must be overridden for client type %s " % self._vcs_type_name)
+        raise NotImplementedError("Base class get_version method must be overridden for client type %s " %
+                                  self._vcs_type_name)
 
-    def checkout(self, url, spec=None, verbose=False, shallow=False):
+    def checkout(self, url, version=None, verbose=False, shallow=False, timeout=None):
         """
         Attempts to create a local repository given a remote
         url. Fails if a target path exists, unless it's an empty directory.
-        If a spec is provided, the local repository
+        If a version is provided, the local repository
         will be updated to that revision. It is possible that
         after a failed call to checkout, a repository still exists,
-        e.g. if an invalid revision spec was given.
+        e.g. if an invalid revision was given.
         If shallow is provided, the scm client may checkout less
         than the full repository history to save time / disk space.
-
+        If a timeout is specified, any pending operation will fail after
+        the specified amount (in seconds). NOTE: this parameter might or
+        might not be honored, depending on VCS client implementation.
         :param url: where to checkout from
         :type url: str
-        :param spec: token for identifying repository revision
-        :type spec: str
+        :param version: token for identifying repository revision
+        :type version: str
         :param shallow: hint to checkout less than a full repository
         :type shallow: bool
+        :param timeout: maximum allocated time to perform operation
+        :type shallow: int
         :returns: True if successful
         """
-        raise NotImplementedError("Base class checkout method must be overridden for client type %s " % self._vcs_type_name)
+        raise NotImplementedError("Base class checkout method must be overridden for client type %s " %
+                                  self._vcs_type_name)
 
-    def update(self, spec=None, verbose=False):
+    def update(self, version=None, verbose=False, timeout=None):
         """
         Sets the local copy of the repository to a version matching
-        the spec. Fails when there are uncommited changes.
+        the version parameter. Fails when there are uncommited changes.
         On failures (also e.g. network failure) grants the
         checked out files are in the same state as before the call.
+        If a timeout is specified, any pending operation will fail after
+        the specified amount (in seconds)
 
-        :param spec: token for identifying repository revision
+        :param version: token for identifying repository revision
            desired.  Token might be a tagname, branchname, version-id,
            SHA-ID, ... depending on the VCS implementation.
+        :param timeout: maximum allocated time to perform operation
+        :type shallow: int
         :returns: True on success, False else
         """
-        raise NotImplementedError("Base class update method must be overridden for client type %s " % self._vcs_type_name)
+        raise NotImplementedError("Base class update method must be overridden for client type %s " %
+                                  self._vcs_type_name)
 
-    def detect_presence(self):
+    @staticmethod
+    def static_detect_presence(path):
         """For auto detection"""
         raise NotImplementedError(
             "Base class detect_presence method must be overridden")
 
+    def detect_presence(self):
+        """For auto detection"""
+        # call static method
+        return self.static_detect_presence(self._path)
+
     def get_vcs_type_name(self):
         """ used when auto detected """
         return self._vcs_type_name
@@ -197,7 +214,8 @@ class VcsClientBase(object):
         :returns: A string summarizing locally modified files
         :rtype: str
         """
-        raise NotImplementedError("Base class get_status method must be overridden for client type %s " % self._vcs_type_name)
+        raise NotImplementedError("Base class get_status method must be overridden for client type %s " %
+                                  self._vcs_type_name)
 
     def get_log(self, relpath=None, limit=None):
         """
@@ -236,4 +254,5 @@ class VcsClientBase(object):
         the extension which will be .tar.gz
         :returns: True on success, False otherwise.
         """
-        raise NotImplementedError("Base class export_repository method must be overridden for client type %s " % self._vcs_type_name)
+        raise NotImplementedError("Base class export_repository method must be overridden for client type %s " %
+                                  self._vcs_type_name)
